// SoapRelayExtension.cs
//
// PRESTO Starter Kit for Microsoft .NET 3.0
// 2007 Microsoft France
// Published under the CECILL-B Free Software license agreement.
// (http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.txt)
// 
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE. 
// THE ENTIRE RISK OF USE OR RESULTS IN CONNECTION WITH THE USE OF THIS CODE 
// AND INFORMATION REMAINS WITH THE USER. 
//

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Description;
using System.ServiceModel.Channels;
using System.ServiceModel.Configuration;
using System.Runtime.Serialization;
using System.Collections.ObjectModel;

namespace Microsoft.Presto.StarterKit.NonNormative
{
    class SoapRelayExtension : IExtension<ServiceHostBase>
    {
        ISoapRelay m_ForwardingChannel = null;
        string m_FinalDestination = null;

        public SoapRelayExtension(string finaldestination)
        {
            m_FinalDestination = finaldestination;
            Uri Via = new Uri(m_FinalDestination);
            EndpointAddress LogicalEndpointAddress = new EndpointAddress("urn:PrestoProxyEndpoint");
                
            NetNamedPipeBinding binding = new NetNamedPipeBinding();
            binding.Security.Mode = NetNamedPipeSecurityMode.None;
            //NetTcpBinding binding = new NetTcpBinding();
            //binding.Security.Mode = SecurityMode.None;

            ChannelFactory<ISoapRelay> channelFactory = new ChannelFactory<ISoapRelay>(binding);

            m_ForwardingChannel = channelFactory.CreateChannel(LogicalEndpointAddress, Via);
            // Add a channel behavior that will turn off validation of @mustUnderstand on the reply's headers.
            //soapRelayfactory.Description.Behaviors.Add(new MustUnderstandBehavior(false));
           // m_ForwardingChannel = soapRelayfactory.CreateChannel();
        }

        public string FinalDestination
        {
            get { return this.m_FinalDestination; }
        }

        public ISoapRelay ForwardingChannel
        {
            get { return this.m_ForwardingChannel; }
        }

        public void Attach(ServiceHostBase owner)
        {

        }

        public void Detach(ServiceHostBase owner)
        {

        }
    }

    class SoapRelayServiceBehavior : Attribute, IServiceBehavior
    {
        private string m_FinalDestination;

        public string FinalDestination
        {
            get { return m_FinalDestination; }
            set { m_FinalDestination = value; }
        }

        #region IServiceBehavior Members

        public void AddBindingParameters(ServiceDescription description, ServiceHostBase serviceHostBase, Collection<ServiceEndpoint> endpoints, BindingParameterCollection parameters)
        {
            
        }

        public void ApplyDispatchBehavior(ServiceDescription description, ServiceHostBase serviceHostBase)
        {
            if (m_FinalDestination == null)
                throw new Exception("Must have a final destination set");
            SoapRelayExtension extension = new SoapRelayExtension(m_FinalDestination);
            serviceHostBase.Extensions.Add(extension);
        }

        public void Validate(ServiceDescription description, ServiceHostBase serviceHostBase)
        {
            
        }

        #endregion
    }

    class MatchAllEndpoints : Attribute, IContractBehavior
    {
        #region IContractBehavior Members

        public void AddBindingParameters(ContractDescription contractDescription, ServiceEndpoint endpoint, BindingParameterCollection bindingParameters)
        {

        }

        public void ApplyClientBehavior(ContractDescription contractDescription, ServiceEndpoint endpoint, ClientRuntime clientRuntime)
        {

        }

        public void ApplyDispatchBehavior(ContractDescription contractDescription, ServiceEndpoint endpoint, DispatchRuntime dispatchRuntime)
        {
            dispatchRuntime.EndpointDispatcher.AddressFilter = new System.ServiceModel.Dispatcher.MatchAllMessageFilter();
        }

        public void Validate(ContractDescription contractDescription, ServiceEndpoint endpoint)
        {

        }

        #endregion
    }
}
